package com.xy.job;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xy.entity.DeviceNetRecord;
import com.xy.entity.SysDictRedis;
import com.xy.mapper.DeviceNetRecordMapper;
import com.xy.utils.DataTime;
import com.xy.utils.JobUtils;
import com.xy.utils.SysDictUtils;
import com.xy.utils.enums.DictEnum;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * <p>
 * 机器-联网记录 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-01-11
 */
@Slf4j
@Component
@AllArgsConstructor
public class DeviceNetRecordJob {

    private DeviceNetRecordMapper deviceNetRecordMapper;

    /**
     * 数据清除
     *
     * @return
     */
    @XxlJob("delDeviceNetRecord")
    public ReturnT<String> delDeviceNetRecord() {
        //数据保留的天数
        SysDictRedis sysDictRedis = SysDictUtils.get(DictEnum.DATA_CLEAR_SIZE.getKey(), "device_net_record_day");
        Integer jobParam = JobUtils.getPrams().gtInt();
        Integer saveDay = jobParam != null ? jobParam
                : sysDictRedis != null ? Integer.valueOf(sysDictRedis.getValue())
                : 30;
        LambdaQueryWrapper<DeviceNetRecord> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceNetRecord>()
                .le(DeviceNetRecord::getCreateTime, DataTime.getStringAround(0, 0, (~(saveDay - 1)), 0, 0, 0))
                .last("limit 10000");
        int delete = deviceNetRecordMapper.delete(lambdaQueryWrapper);
        if (delete > 0) {
            log.info("机器联网记录清除完成，共{}条", delete);
        }
        return ReturnT.SUCCESS;
    }
}
