package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.DeviceStatusDto;
import com.xy.entity.DeviceStatus;
import com.xy.mapper.DeviceStatusMapper;
import com.xy.mapper.entity.DeviceStatusCount;
import com.xy.utils.Emptys;
import com.xy.utils.MybatisPlusQuery;
import com.xy.utils.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.List;

import static com.xy.utils.Beans.copy;

/**
 * <p>
 * 设备-状态 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2022-12-23
 */
@Service
@Api(tags = "设备-状态")
public class DeviceStatusServiceImpl extends ServiceImpl<DeviceStatusMapper, DeviceStatus> implements DeviceStatusService {

    @Override
    @ApiOperation("上报信息")
    public R up(DeviceStatusDto.Up up) {
        DeviceStatus deviceStatusInfo = getById(up.getDeviceId());
        if (deviceStatusInfo == null) {
            return R.ok();
        }
        DeviceStatus deviceStatus = copy(DeviceStatus.class, up).setUpdateTime(LocalDateTime.now());
        updateById(deviceStatus);
        return R.ok();
    }

    @PostMapping("obj")
    @ApiOperation("对象查询")
    public R<DeviceStatusDto.Vo> obj(@RequestBody DeviceStatusDto.Vo vo) {
        LambdaQueryWrapper<DeviceStatus> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(vo, DeviceStatus.class).build();
        List<DeviceStatus> list = list(lambdaQueryWrapper);
        if (!Emptys.check(list)) {
            return R.ok();
        }
        return R.ok(copy(DeviceStatusDto.Vo.class, list.get(0)));
    }

    @PostMapping("list")
    @ApiOperation("集合查询")
    public R<List<DeviceStatusDto.Vo>> list(@RequestBody DeviceStatusDto.SelectList selectList) {
        LambdaQueryWrapper<DeviceStatus> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList, DeviceStatus.class)
                .in(DeviceStatus::getDeviceId, selectList.getDeviceIds())
                .build();
        List<DeviceStatus> list = list(lambdaQueryWrapper);
        return R.ok(copy(DeviceStatusDto.Vo.class, list));
    }

    @PostMapping("statusCount")
    @ApiOperation("统计状态数量")
    public R<DeviceStatusCount> statusCount() {
        DeviceStatusCount deviceStatusCount = baseMapper.statusCount();
        return R.ok(deviceStatusCount);
    }
}
